/*
Example for G2C - Weather Station

    Date          : June 2019
    Author        : Nemanja Medakovic

Test configuration STM32 :

    MCU              : STM32F407ZG
    MCU Clock        : 168MHz
    Dev. Board       : Fusion for STM32 (ARM) v8
    TFT              : 5"
    ARM Compiler ver : v6.2.0.0

---

Description :

The application is composed of three sections :

- System Initialization - Initializes all necessary GPIO pins, TFT,
and I2C used for the communication with others sensors.
- Application Initialization - Initializes drivers, power on module.
All Click boards will be initialized in this function.
- Application Task - If weather measurement was changed, all measurement results will be showed
on TFT screen.

*/

#include "Click_Ambient6_types.h"
#include "Click_RTC_7_types.h"
#include "Click_RTC_7_config.h"
#include "Click_TempHum8_types.h"
#include "Click_UV_4_types.h"
#include "Click_UV_4_config.h"
#include "MyProject_objects.h"
#include "MyProject_resources.h"

/** Control variables to enable/disable data update */
static uint8_t state;
static uint8_t oldstate;

static uint16_t tx_idx;
static uint8_t first_write;

static TImage * image;
static uint8_t * font;

/** Data where stored the previous measurement results (needed for data update commparing) */
static uint8_t prev_sec;
static uint8_t prev_min;
static uint8_t prev_hour;
static uint8_t prev_week;
static uint8_t prev_day;
static uint8_t prev_month;
static uint8_t prev_year;
static int8_t prev_temp;
static uint8_t prev_hum;
static uint16_t prev_light;
static uint16_t prev_uv;
static uint16_t prev_ir;

/** Data measurement results */
static rtc7_time_t timeSet;
static rtc7_time_t timeDate;
static int8_t temperature;
static uint8_t humidity;
static uint16_t ambientLight;
static uint16_t uv_data[ 6 ];

static const uint8_t _MATCHED     = 0;
static const uint8_t _NOT_MATCHED = 1;

void _strcpy( uint8_t *_from, uint8_t *_to )
{
    while (*_from != 0)
    {
        *_to = *_from;
        _to++;
        _from++;
        tx_idx++;
    }

    *_to = 0;
}

uint16_t _strlen( uint8_t *_str )
{
    uint16_t length = 0;

    while (*_str != 0)
    {
        length++;
        _str++;
    }

    return length;
}

void displayCtrl()
{
    if ( Button( &GPIOA_IDR, 0, 1, 1 ) )
    {
        state = !state;
    }
}

void initTFT()
{
    uint16_t i;

    TFT_Init_SSD1963_Board_50( 800, 480 );
    TFT_Set_DBC_SSD1963( 0 );
    TFT_Set_Pen( 0xDC40, 1 );
    TFT_Set_Font( Montserrat41x51_Regular, 0xDC40, FO_HORIZONTAL );
    Start_TP();
    TFT_Fill_Screen( CL_BLACK );
    TFT_H_Line( 0, 799, 0 );
    TFT_V_Line( 1, 479, 799 );
    TFT_H_Line( 798, 0, 479 );
    TFT_V_Line( 478, 1, 0 );
    TFT_Write_Text( "Initializing", 320, 230 );

    /** Display turn ON effect */
    for (i = 1; i < 256; i++)
    {
        TFT_Set_DBC_SSD1963( i );
        Delay_ms( 2 );
    }
}

void waitingProcess()
{
    uint16_t i;
    uint16_t j;
    uint16_t k;

    for (k = 0; k < 9; k++)
    {
        j = 0;
        for (i = 0; i < 800; i++)
        {
            TFT_Set_Pen( CL_BLACK, 1 );
            TFT_H_Line( j, j + 92, 1 );

            TFT_Set_Pen( 0xDC40, 1 );
            TFT_H_Line( i, i + 30, 1 );
            TFT_Set_Pen( CL_YELLOW, 1 );
            TFT_H_Line( i + 31, i + 61, 1 );
            TFT_Set_Pen( CL_WHITE, 1 );
            TFT_H_Line( i + 62, i + 92, 1 );

            j = i;
            Delay_ms( 1 );
        }

        j = 0;
        for (i = 0; i < 800; i++)
        {
            TFT_Set_Pen( CL_BLACK, 1 );
            TFT_H_Line( j, j + 92, 478 );

            TFT_Set_Pen( 0xDC40, 1 );
            TFT_H_Line( i, i + 30, 478 );
            TFT_Set_Pen( CL_YELLOW, 1 );
            TFT_H_Line( i + 31, i + 61, 478 );
            TFT_Set_Pen( CL_WHITE, 1 );
            TFT_H_Line( i + 62, i + 92, 478 );

            j = i;
            Delay_ms( 1 );
        }
    }
}

void updateLabel( uint8_t * new_label, uint16_t x_pos, uint16_t y_pos )
{
    DrawImage( image );
    TFT_Set_Font( font, CL_WHITE, FO_HORIZONTAL );
    TFT_Write_Text( new_label, x_pos, y_pos );
}

void systemInit()
{
    /** GPIO initialization for RTC 7 click */
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_INT_PIN, _GPIO_INPUT );
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_AN_PIN,  _GPIO_INPUT );
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_RST_PIN, _GPIO_INPUT );

    /** GPIO initialization for UV 4 click */
    mikrobus_gpioInit( _MIKROBUS3, _MIKROBUS_INT_PIN, _GPIO_INPUT );

    /** Button used for display enable/disable control */
    GPIO_Digital_Input( &GPIOA_BASE, _GPIO_PINMASK_0 );

    /** GPIO initialization for LED signalization */
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_8 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_9 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_10 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_11 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_12 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_13 );
    GPIOC_ODR.B8 = 0;
    GPIOC_ODR.B9 = 0;
    GPIOC_ODR.B10 = 0;
    GPIOC_ODR.B11 = 0;
    GPIOC_ODR.B12 = 0;
    GPIOC_ODR.B13 = 0;

    /** I2C initialization for UV 4, Ambient 6 and Temp-Hum 8 clicks */
    mikrobus_i2cInit( _MIKROBUS3, &_UV4_I2C_CFG[0] );
    Delay_ms( 100 );

    /** I2C initialization for RTC 7 click */
    mikrobus_i2cInit( _MIKROBUS4, &_RTC7_I2C_CFG[0] );
    Delay_ms( 100 );

    /** TFT init */
    initTFT();
}

void applicationInit()
{
    uint16_t i;

    /** Temp-Hum 8 init */
    I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );

    temphum8_i2cDriverInit( (T_TEMPHUM8_P)&_MIKROBUS1_GPIO, (T_TEMPHUM8_P)&_MIKROBUS1_I2C, 0x40 );
    Delay_ms( 300 );
    temphum8_softReset();
    Delay_ms( 200 );
    temphum8_configuration( _TEMPHUM8_CFG_RESOLUTION_H12_T14 | _TEMPHUM8_CFG_ENABLE_ON_CHIP_HEATER | _TEMPHUM8_CFG_DISABLE_OTP_RELOAD );

    /** Ambient 6 init */
    ambient6_i2cDriverInit( (T_AMBIENT6_P)&_MIKROBUS2_GPIO, (T_AMBIENT6_P)&_MIKROBUS2_I2C, _AMBIENT6_DEVICE_SLAVE_ADDRESS );
    Delay_ms( 300 );
    ambient6_configuration( _AMBIENT6_CFG_ALS_SENSITIVITY_X2 |
                            _AMBIENT6_CFG_ALS_TIME_SETTING_25MS |
                            _AMBIENT6_CFG_PERSISTENCE_NUMBER_1 |
                            _AMBIENT6_CFG_ALS_INT_DISABLE |
                            _AMBIENT6_CFG_ALS_POWER_ON );
    ambient6_writeData( _AMBIENT6_CMD_MODE_SETTINGS, _AMBIENT6_POWER_MODE_1 | _AMBIENT6_POWER_MODE_ENABLE );
    Delay_ms( 200 );
    TFT_Write_Text( ".", 505, 230 );

    /** UV 4 init */
    uv4_i2cDriverInit( (T_UV4_P)&_MIKROBUS3_GPIO, (T_UV4_P)&_MIKROBUS3_I2C, 0x55 );
    Delay_ms( 300 );
    uv4_writeReg( _UV4_COMMAND_REG, _UV4_RESET_CTR_CMD );
    Delay_ms( 200 );
    uv4_writeParamData( _UV4_CHAN_LIST_PARAM,    _UV4_CHAN0_ENABLE | _UV4_CHAN1_ENABLE );
    uv4_writeParamData( _UV4_ADCPOST_0_PARAM,    _UV4_16BIT_OUT );
    uv4_writeParamData( _UV4_ADCPOST_1_PARAM,    _UV4_16BIT_OUT );
    uv4_writeParamData( _UV4_MEASCONFIG_0_PARAM, _UV4_MEASCOUNT1_SELECT );
    uv4_writeParamData( _UV4_MEASCONFIG_1_PARAM, _UV4_MEASCOUNT1_SELECT );
    uv4_writeParamData( _UV4_ADCCONFIG_0_PARAM,  _UV4_OPT_FUNC_UV );
    uv4_writeParamData( _UV4_ADCCONFIG_1_PARAM,  _UV4_OPT_FUNC_SMALL_IR );
    uv4_writeParamData( _UV4_BURST_PARAM,        _UV4_BURST_DISABLE );
    uv4_writeParamData( _UV4_MEASRATE_H_PARAM,   0 );
    uv4_writeParamData( _UV4_MEASRATE_L_PARAM,   0 );
    uv4_writeParamData( _UV4_MEASCOUNT_1_PARAM,  0 );

    /** RTC 7 init */
    I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );

    /** If you want to reset RTC 7 click on the selected time, you need to uncomment 'timeSet' part of code */
    timeSet.seconds  = 30;
    timeSet.minutes  = 16;
    timeSet.hours    = 14;
    timeSet.weekdays = 3;
    timeSet.monthday = 9;
    timeSet.month    = 10;
    timeSet.year     = 19;

    rtc7_i2cDriverInit( (T_RTC7_P)&_MIKROBUS4_GPIO, (T_RTC7_P)&_MIKROBUS4_I2C, 0x69 );
    Delay_ms( 500 );
    /*rtc7_reset();
    rtc7_disableDst();
    rtc7_init( 0, _RTC7_24HR_FORMAT );
    rtc7_setGmtTime( &timeSet );*/
    rtc7_setOsc( _RTC7_ENABLE_OSC, _RTC7_INPUT_FREQ_32768HZ, _RTC7_OUTPUT_FREQ_32768HZ );
    rtc7_setTimer( _RTC7_TIMER_EN, _RTC7_TIMER_FREQ_16HZ );
    TFT_Write_Text( ".", 520, 230 );

    waitingProcess();

    state    = 0;
    oldstate = 0;

    prev_sec   = 60;
    prev_min   = 60;
    prev_hour  = 24;
    prev_week  = 0;
    prev_day   = 0;
    prev_month = 0;
    prev_year  = 100;
    prev_temp  = 200;
    prev_hum   = 101;
    prev_light = 65535;
    prev_uv    = 65535;
    prev_ir    = 65535;

    /** Start Work Screen */
    for (i = 255; i > 0; i--)
    {
        TFT_Set_DBC_SSD1963( i );
        Delay_ms( 3 );
    }
    TFT_Set_DBC_SSD1963( 0 );

    DrawScreen( &Screen2 );

    /** Ready signalization */
    for (i = 0x0100; i < 0x4000; i <<= 1)
    {
        GPIOC_ODR |= i;
        Delay_ms( 20 );
    }
    for (i = 0x0100; i < 0x4000; i <<= 1)
    {
        GPIOC_ODR ^= i;
        Delay_ms( 20 );
    }

    /** Show TFT Screen */
    for (i = 0; i < 256; i++)
    {
        TFT_Set_DBC_SSD1963( i );
        Delay_ms( 3 );
    }
}

void applicationTask()
{
    /** Button A0 can enable or disable the results updating */
    displayCtrl();

    /** INIT VALUES FOR CONTROL VARIABLES, START MEASUREMENT */
    if ( (state == 1) && (oldstate == 0) )
    {
        uint16_t i;

        prev_sec   = 60;
        prev_min   = 60;
        prev_hour  = 24;
        prev_week  = 0;
        prev_day   = 0;
        prev_month = 0;
        prev_year  = 100;
        prev_temp  = 200;
        prev_hum   = 101;
        prev_light = 65535;
        prev_uv    = 65535;
        prev_ir    = 65535;

        oldstate    = 1;
        first_write = 1;
    }

    /** TFT AND CLOUD MEASUREMENT RESULTS UPDATE */
    if ( (state == 1) && (oldstate == 1) )
    {
        rtc7_getLocalTime( &timeDate );

        if (prev_sec != timeDate.seconds)
        {
            char tx_tmp[ 20 ];
            char tx_lab[ 50 ];

            /** TIME UPDATE */
            tx_idx = 0;
            prev_sec = timeDate.seconds;
            ByteToStr( timeDate.hours, tx_tmp );
            Ltrim( tx_tmp );
            if (prev_hour != timeDate.hours)
            {
                prev_hour = timeDate.hours;
            }
            if (_strlen( tx_tmp ) == 1)
            {
                tx_tmp[ 1 ] = tx_tmp[ 0 ];
                tx_tmp[ 0 ] = '0';
                tx_tmp[ 2 ] = 0;
            }
            _strcpy( &tx_tmp[0], &tx_lab[0] );
            _strcpy( " : ", &tx_lab[ tx_idx ] );
            if (prev_min != timeDate.minutes)
            {
                ByteToStr( timeDate.minutes, tx_tmp );
                Ltrim( tx_tmp );

                prev_min = timeDate.minutes;

                if (_strlen( tx_tmp ) == 1)
                {
                    tx_tmp[ 1 ] = tx_tmp[ 0 ];
                    tx_tmp[ 0 ] = '0';
                    tx_tmp[ 2 ] = 0;
                }
                _strcpy( &tx_tmp[0], &tx_lab[ tx_idx ] );
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                font = Montserrat_SemiBold65x78_Regular;
                image = &Image2;
                updateLabel( tx_lab, 258, 87 );
            }

            if (prev_day != timeDate.monthday)
            {
                /** DATE UPDATE */
                tx_idx = 0;
                prev_day = timeDate.monthday;
                ByteToStr( timeDate.month, tx_tmp );
                Ltrim( tx_tmp );
                if (prev_month != timeDate.month)
                {
                    prev_month = timeDate.month;
                }
                _strcpy( &tx_tmp[0], &tx_lab[0] );
                _strcpy( " / ", &tx_lab[ tx_idx ] );
                ByteToStr( timeDate.monthday, tx_tmp );
                Ltrim( tx_tmp );
                _strcpy( &tx_tmp[0], &tx_lab[ tx_idx ] );
                _strcpy( " / ", &tx_lab[ tx_idx ] );
                ByteToStr( timeDate.year, tx_tmp );
                Ltrim( tx_tmp );
                if (prev_year != timeDate.year)
                {
                    prev_year = timeDate.year;
                }
                _strcpy( &tx_tmp[0], &tx_lab[ tx_idx ] );
                if (prev_week != timeDate.weekdays)
                {
                    prev_week = timeDate.weekdays;
                    ByteToStr( timeDate.weekdays, tx_tmp );
                    Ltrim( tx_tmp );
                }
                switch (timeDate.weekdays)
                {
                    case 1 :
                    {
                        _strcpy( "    MON", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 2 :
                    {
                        _strcpy( "    TUE", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 3 :
                    {
                        _strcpy( "    WED", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 4 :
                    {
                        _strcpy( "    THU", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 5 :
                    {
                        _strcpy( "    FRI", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 6 :
                    {
                        _strcpy( "    SAT", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 7 :
                    {
                        _strcpy( "    SUN", &tx_lab[ tx_idx ] );
                    }
                }
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                font = Montserrat41x51_Regular;
                image = &Image3;
                updateLabel( tx_lab, 292, 173 );
            }

            I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );
            font = Montserrat41x51_Regular;

            humidity = temphum8_getHumidity();

            if (prev_hum != humidity)
            {
                /** HUMIDITY UPDATE */
                prev_hum = humidity;
                ByteToStr( humidity, tx_tmp );
                Ltrim( tx_tmp );
                I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                image = &Image4;
                updateLabel( tx_tmp, 136, 342 );
                I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );
            }

            temperature = temphum8_getTemperature( _TEMPHUM8_TEMPERATURE_IN_CELSIUS );

            if (prev_temp != temperature)
            {
                /** TEMPERATURE UPDATE */
                prev_temp = temperature;
                ShortToStr( temperature, tx_tmp );
                Ltrim( tx_tmp );
                I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                image = &Image5;
                updateLabel( tx_tmp, 383, 342 );
                I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );
            }

            uv4_readMeasurements( &uv_data[0] );

            if (prev_ir != uv_data[1])
            {
                /** IR LIGHT UPDATE */
                prev_ir = uv_data[1];
                WordToStr( uv_data[1], tx_tmp );
                Ltrim( tx_tmp );
                I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                image = &Image6;
                updateLabel( tx_tmp, 660, 344 );
                I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );
            }

            if (prev_uv != uv_data[0])
            {
                /** UV LIGHT UPDATE */
                prev_uv = uv_data[0];
                WordToStr( uv_data[0], tx_tmp );
                Ltrim( tx_tmp );
                I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                image = &Image7;
                updateLabel( tx_tmp, 660, 204 );
                I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );
            }

            ambientLight = ambient6_getAmbientLight();
            I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );

            if (prev_light != ambientLight)
            {
                /** AMBIENT LIGHT UPDATE */
                prev_light = ambientLight;
                WordToStr( ambientLight, tx_tmp );
                Ltrim( tx_tmp );
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                    first_write = 0;
                }
                image = &Image8;
                updateLabel( tx_tmp, 660, 79 );
            }
        }
    }

    /** Draw screen on display without measurement results */
    if ( (state == 0) && (oldstate == 1) )
    {
        uint16_t i;

        for (i = 254; i > 0; i--)
        {
            TFT_Set_DBC_SSD1963( i );
            Delay_ms( 2 );
        }
        TFT_Set_DBC_SSD1963( 0 );

        DrawScreen( &Screen2 );

        for (i = 1; i < 256; i++)
        {
            TFT_Set_DBC_SSD1963( i );
            Delay_ms( 2 );
        }

        oldstate = 0;
    }
}

void main()
{
    systemInit();
    applicationInit();

    while (1)
    {
        applicationTask();
    }
}

/* -------------------------------------------------------------------------- */